
% This MATLAB code prepares PSG inputs for the optimization problem with SSD constraints, exports the problem to txt format, 
% solves the problem using mpsg_solver function.
%
% Here are main steps:
%
% loading the following data:
%   1) matrix_sde.txt containing return scenarios;
%   2) vector_benchmark_sd.txt containing returns of the benchmark;
%
% preparing right hand-sides for SSD constraints;
% 
% preparing the problem statement;
%
% exporting the problem to txt format;
%
% solving the problem using mpsg_solver function;
%
% displaing rezults.


%% 0. Clear command window and workspace
clc; clear

%% 1. Load problem data
disp(' ');
disp(' Part 1. Load problem data and generate problem statement ')
disp(' ')

% Data for Dow Jones index
matrix_sde=load('matrix_sde_0.txt'); 
benchmark_sd=load('vector_benchmark_sd_0.txt'); 

iargstruc_arrn=struct();
iargstruc_arrn(1).name='matrix_sde';
J=size(matrix_sde,1);   % A number of scenarios
I=size(matrix_sde,2);     % A number of variables

iargstruc_arrn(1).vars='';
for i=1:I
  iargstruc_arrn(1).vars=[iargstruc_arrn(1).vars,char(9),'x',int2str(i)];
end
iargstruc_arrn(1).data=matrix_sde;

iargstruc_arrn(2).name='matrix_budget';
iargstruc_arrn(2).vars=iargstruc_arrn(1).vars;
iargstruc_arrn(2).data=ones(I,1)';


%cd('..');

%% Ubound preparation 

n=length(benchmark_sd);
ub=zeros(n,1);
ubound_sd=zeros(n,1);
for j=1:n; w=benchmark_sd(j);
    for i=1:n
      ub(i)= max( w - benchmark_sd(i),0);
    end
    ubound_sd(j)=mean(ub);
end
clear i j ub 

%% 2.1 Form Vectors for MultiConstraint

count=2; 
count=count+1;
iargstruc_arrn(count) = object_pack('vector_ubound_sd','Value',ubound_sd); 
count=count+1;
iargstruc_arrn(count) = object_pack('vector_benchmark_sd','Value',-benchmark_sd); 

%% 3. Form problem_statement

problem_head=['maximize', char(10),...
              '  avg_g(matrix_sde)', char(10),...
              'Constraint: = 1', char(10),...
              '  linear(matrix_budget)',char(10)];
problem_newc=['MultiConstraint: <= vector_ubound_sd',char(10),...
              '  pm_pen(vector_benchmark_sd, matrix_sde)',char(10)];
problem_box= ['Box: >= 0, <= 1'];

problem_statement=[problem_head, problem_newc, problem_box];


clear problem_head problem_newc problem_box j j1 ic;


%% 4. Export problem to txt format

disp(' '); disp(' ')
disp(' Part 4. Export problem to txt format')
disp(' ')

mpsg_problem_exporttotext(problem_statement, iargstruc_arrn,'.');

%% 5. Solve problem

disp(' '); disp(' ')
disp(' Part 5. Solve problem ')
disp(' ')

%tic
[solution_SSD, outargstruc_SSD] = mpsg_solver(problem_statement, iargstruc_arrn);
%toc   %prints the elapsed time since tic was used.


%% 6. Report

disp(' '); disp(' ')
disp(' Part 6. Report ')
disp(' ')
disp('Solution_SSD ')
dx = strfind(solution_SSD, 'Constraint:');
disp(solution_SSD(1:dx(1)-1));
clear dx;
for i=1:length(outargstruc_SSD); 
    if strcmp(outargstruc_SSD(i).object_name(1:13),'point_problem')==1, Optimal_point=transpose(outargstruc_SSD(i).values); end
end    
disp('Optimal point:');
for i=1:I;  disp(['x', int2str(i),' = '  num2str(Optimal_point(i),7)]);
end
clear i; disp(' ')
